@@ -459,7 +459,7 @@ class Frame(FrameSummary, Value):
         self.job: Job = job
 
         # Advance past decorator.
-        if self.line.strip().startswith("@"):
+        if self._line and self._line.strip().startswith("@"):
             while True:
                 self.lineno += 1
                 line = linecache.getline(self.filename, self.lineno).strip()
@@ -706,7 +706,7 @@ class Scheduler:
         Add executor to scheduler.
         """
         self.executors[executor.name] = executor
-        executor.scheduler = self
+        executor.set_scheduler(self)
 
     def load(self, migrate: Optional[bool] = None) -> None:
         self.backend.load(migrate=migrate)
